#!/usr/bin/perl
#
# LightSquid Project (c) 2004-2005 Sergey Erokhin aka ESL
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# detail see in gnugpl.txt

#parse access.log
# make per user report in 'report' direcotry

#usage: lightparse.pl {param}
#if param omit               - parse full access.log file
#   today                    - only current day
#   data in format YYYYMMDD  - parse day
#   access.log.{\d}.{gz|bz2} - parse file (for process archived)

use File::Basename;
push (@INC,(fileparse($0))[1]);

require "lightsquid.cfg";
require "common.pl";

#include ip2name function
require "ip2name/ip2name.$ip2name";

# function prototypes
sub MakeReport();

my $skipurlcntr       = 0;
my $skip4xxcntr       = 0;
my $skipfilterdatecntr= 0;

my $firstrun    = 1;
my $totallines  = 0;
my $parsedlines = 0;
my $daylines    = 0;

my $catname   ="cat";
my $filename  ="access.log";

undef $workday;

if ($skipurl eq "") {
   $skipurl = "skipurl MUST be defined!!!";
   print "WARNING !!! \$skipurl is empty\n";
}   

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime;
$month=sprintf("%02d",$mon+1);;
      
if ($ARGV[0] eq "today") {
   ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime;
   $filterdate=sprintf("%04d%02d%02d",$year+1900,$mon+1,$mday);;
   print ">>> filter today: $filterdate\n" if ($debug);
}

if ($ARGV[0] =~ m/^\d\d\d\d\d\d\d\d$/) {
   $filterdate=$ARGV[0];
   print ">>> filter date:  $filterdate\n" if ($debug);
}

if ($ARGV[0] =~ m/access\.log\.(\d)/) {
   $filename=$ARGV[0];
   $catname="zcat" if ($ARGV[0] =~ m/\.gz$/);
   $catname="bzcat" if ($ARGV[0] =~ m/\.bz2$/);
}

print ">>> use file :: $filename\n" if ($debug);

open FF, "$catname $logpath/$filename|" || die "can't access log file\n";


StartUser();

undef %bigfile; $bigfilecnt=0;
while (<FF>) {
    chomp;
    $totallines++;

    if (0 == $squidlogtype) {
       #squid native log
       #970313965.619 1249    denis.local TCP_MISS/200 2598 GET    http://www.emalecentral.com/tasha/thm_4374x013.jpg -     DIRECT/www.emalecentral.com image/jpeg
       # timestamp    elapsed host        type         size method url                                                  user  hierarechy                    type
       ($Ltimestamp,$Lelapsed,$Lhost,$Ltype,$Lsize,$Lmethod,$Lurl,$Luser,$Lhierarchy,$Lconttype)=split;
       ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($Ltimestamp);
       $mon++; #fix, month start from 0
    } else {
       #emulated httpd log
       #192.168.3.40 - - [15/Apr/2005:11:46:35 +0300] "GET http://mail.yandex.ru/mboxjscript? HTTP/1.0" 200 2262 TCP_MISS :DIRECT
       #192.168.3.40 - - [15/Apr/2005:11:46:35 +0300] "GET http://css.yandex.ru/css/mail/search.js HTTP/1.0" 200 4199 TCP_HIT:NONE
       #192.168.3.12 -       -       [15/Apr/2005:11:46:35 +0300] "CONNECT aero.lufthansa.com:443 HTTP/1.0" 200 35992 TCP_MISS:DIRE
       # ($Lhost,     $Luser,$Luser2,$Ldate,               $u2,   $Lmethod,$Lurl,                 $u3,     $Ltype,$Lsize,$u4)=split
 
       ($Lhost,$Luser,$Luser2,$Ldate,$u2,$Lmethod,$Lurl,$u3,$Ltype,$Lsize,$u4)=split;
       
       $Ldate =~ m#^\[(\d\d)/(...)/(\d\d\d\d):(\d\d):(\d\d):(\d\d)#;
       $mday=$1;$mon=$month2dec{$2};$year=$3-1900;
       $hour=$4;$min=$5;$sec=$6;
       
       if (($Luser eq "-") && ($Luser2 ne "-")) {
         $Luser = $Luser2;
       }
       
       $u4 =~ m/(.*?)\s?:(.*)/;
       $Ltype = "$1/$Ltype";
    }  
    
    if ($year < 2000-1900) { ; #invalid record
       print ">>>> skipn Bad Year  $Lurl\n$_" if ($debug2 >= 1 );
       $skipbadyear++;
       next;
    }
    
    $date  =sprintf("%04d%02d%02d",$year+1900,$mon,$mday);
    if ($filterdate) {
       if ($date ne $filterdate) {
          print ">>>> skipDafteFilter URL $Lurl\n$_" if ($debug2 >= 2 );
          $skipfilterdatecntr++;
          next;
       };
    }

    #skip intranet
    if ($Lurl =~ m/$skipurl/o) {
      print ">>>> skipURL $Lurl\n$_" if ($debug2 >= 2 );
      $skipurlcntr++;
      next;
    };

    # skip Access denied records (TODO: report)
    if ($Ltype =~ m#DENIED#io) {
      $skipDenied++;
      print ">>>> skipDenied $Ltype\n$_" if ($debug2 >= 2);
      next;
    };
    
    if ($Ltype =~ m/HIT/) {
      $CacheHIT+=$Lsize;
    } else {
      $CacheMISS+=$Lsize;
    }

    $parsedlines++;
    
    if ($date ne $workday) { # close prev day, prepare for new
      if ($firstrun) {
        undef $firstrun;
        $workday=$date;
      } else {
        MakeReport();
        undef %totalsize; undef %sitesize; undef %sitehit;undef %totalhit; 
	undef %hashhost;undef %hashname;
        undef %bigfile; $bigfilecnt=0;
	undef %sitetime;undef %sitetimesize;
        $daylines=0;
        $workday=$date;
	$sqlreq=0;
	$CacheHIT=0;$CacheMISS=0;
      }
    }
    $daylines++;

    $user=lc $Luser;
    
    # if username is defined, use it, else use IP
    
    if ($user eq "-") {
      #defined in lightsquid.cfg
      $user=GetNameByIP($Lhost);
      next if ($user eq "SKIP THIS IP");
    }

    #simplified some common banner system & counters
    $url=$Lurl;
    $url =~ s/([a-z]+:\/\/)??.*\.(spylog\.com)/$1www.$2/o;
    $url =~ s/([a-z]+:\/\/)??.*\.(yimg\.com)/$1www.$2/o;
    $url =~ s/([a-z]+:\/\/)??.*\.(adriver\.ru)/$1www.$2/o;
    $url =~ s/([a-z]+:\/\/)??.*\.(bannerbank\.ru)/$1www.$2/o;
    $url =~ s/([a-z]+:\/\/)??.*\.(mail\.ru)/$1www.$2/o;
    $url =~ s/([a-z]+:\/\/)??.*\.(adnet\.ru)/$1www.$2/o;

    #extract site name
    $url =~ m/([a-z]+:\/\/)??([a-z0-9\-]+\.){1}(([a-z0-9\-]+\.){0,})([a-z0-9\-]+){1}(:[0-9]+)?\/(.*)/o;
    $site=$2.$3.$5;
    
    $site=$Lurl if ($site eq "");

    $totalsize{$user}       +=$Lsize;
    $totalhit {$user}       ++;
    $sitesize {$user}{$site}+=$Lsize;
    $sitehit  {$user}{$site}++;
    
    $sitetime     {$user}{$site}[$hour]+=$Lelapsed;
    $sitetimesize {$user}{$site}[$hour]+=$Lsize;
    
    #.bigfile support
    if ($Lsize > $bigfilelimit) {
        $bigfile [$bigfilecnt]{date}=sprintf("%02d:%02d:%02d",$hour,$min,$sec);
        $bigfile [$bigfilecnt]{link}=$Lurl;
        $bigfile [$bigfilecnt]{size}=$Lsize;
        $bigfile [$bigfilecnt]{user}=$user;
        $bigfilecnt++;
    }
}

MakeReport();

StopUser();


if ($debug) {
  print  "run TIME: ".(time()-$^T)." sec\n";
  print  "LightSquid parser statistic report\n\n";
  printf ("      %10d lines processed\n",$totallines);
  printf ("      %10d lines parsed\n",$parsedlines);
  printf ("      %10d lines skiped by bad year\n",$skipbadyear);
  printf ("      %10d lines skiped by date filter\n",$skipfilterdatecntr);
  printf ("      %10d lines skiped by Denied filter\n",$skipDenied);
  printf ("      %10d lines skiped by skipURL filter\n",$skipurlcntr);

  if ( $parsedlines == 0 ) {
    print "\nWARNING !!!!, parsed 0 lines from total : $totallines\n";
    print "please check confiuration !!!!\n";
    print "may be wrong log format selected ?\n";
  }
  
}

##Subroutines

sub MakeReport() {
        #generate report
        #use global var 

        return if ($daylines < 2);

        print ">>> Make Report $workday ($daylines - log line parsed)\n" if ($debug);

        $reppath="$reportpath/$workday";

        unless ( -d $reppath )
        {
          mkdir $reppath, 0755 or die "Can't create dir '$reppath': $!";
        }
	

        open TOTALFILE,">$reppath/.total" || die "can't create file  $reppath/.total - $!";

        $tmp="";$tmpsize=0;$tmpuser=0;$tmpoveruser=0;

        foreach $tuser (sort {$totalsize{$b} <=> $totalsize{$a}} keys %totalsize) {
          $tmp.="$tuser\t$totalsize{$tuser}\t$totalhit{$tuser}\n";
          $tmpuser++;
          $tmpsize+=$totalsize{$tuser};
	  $tmpoveruser++ if ($totalsize{$tuser} >= $perusertrafficlimit);

          open REPFILE,">$reppath/$tuser" || die "can't create file  $reppath/$tuser - $!";

          print REPFILE "total: $totalsize{$tuser}\n";

          foreach $tsite (sort {$sitesize{$tuser}{$b} <=> $sitesize{$tuser}{$a}} keys %{$sitesize{$tuser}} ) {
              printf REPFILE ("%-25s %10d %5d\t",$tsite,$sitesize{$tuser}{$tsite},$sitehit{$tuser}{$tsite});
	      if ($timereport != 0) {
                for ($hour=0;$hour<24;$hour++) {
  	          printf REPFILE ("%d-%d ",int($sitetime{$tuser}{$tsite}[$hour]/3600),$sitetimesize{$tuser}{$tsite}[$hour]);
	        }
	      }
              print REPFILE "\n";
	      
	      
          }
          close REPFILE;
        }
	
	$CacheMISS=1 if ($CacheMISS == 0);

        print TOTALFILE "user: $tmpuser\n";
        print TOTALFILE "size: $tmpsize\n";
	
        print TOTALFILE "$tmp";
        close TOTALFILE;

        open FILE,">$reppath/.features" || die "can't create file  $reppath/.features - $!";
        print FILE "overuser: $tmpoveruser\n";
        print FILE "cachehit%: ".sprintf("%3.2f",($CacheHIT*100)/($CacheHIT+$CacheMISS))."\n";
        print FILE "cachehit: $CacheHIT\n";
        print FILE "cachemiss: $CacheMISS\n";
        print FILE "cacheall: ".($CacheHIT+$CacheMISS)."\n";
	close FILE;

        unlink "$reppath/.bigfiles";
	#        print "clear hash\n";
        if ($bigfilecnt != 0) {
          open MAXFILE,">$reppath/.bigfiles" || die "can't create file  $reppath/.maxfiles - $!";
          for ($i=0;$i<$bigfilecnt;$i++) {
            print MAXFILE "$bigfile[$i]{user}\t$bigfile[$i]{date}\t$bigfile[$i]{size}\t$bigfile[$i]{link}\n";
          }
          close MAXFILE;
        }
	CreateGroupFile($reppath);
	CreateRealnameFile($reppath);
#	print ">>> done \n" if ($debug);
}

__END__
2004-04-23     	: initial version
2004-09-01 FIX 	: error in parse invalid file
2004-09-09 ADD 	: add create .bigfile file contain links greater $bigfilelimit
2004-11-08 ADD 	: skip 4xx records (dirty :-() TODO: do error report
2004-11-09 ADD 	: use DB only if not define user name...
2005-04-13 ADD 	: LightSquid publication cleanup
2005-04-14 ADD 	: $debug and $debug2 variable for generate statistic
                : if parsed lines = 0 print WARNING
2005-04-17 ADD 	: add support fot HTTPDlike log file
2005-04-19 ADD 	: add .bz2 support
        	: add cache hit calculationn (if Ltype contain HIT - hit else - MISS), wrong ??
                : add oversized user calculation
2005-04-20 ADD 	: .features file added, with additional info	       
2005-04-22 FIX 	: httpdlike parser bug;
	   FIX	: mkdir 655 -> mkdir 755
	   
2005-04-30 ADD  : Rewrite archive support, now support access.log.{D},access.log.{D}.gz,access.log.{D}.bz2
           ADD  : time report

2005-05-03 FIX  : fix wrong .features file output
2005-05-12 FIX  : empty line report only if $debug
           FIX  : date filter now ^\d\d\d\d\d\d\d\d$ ...	   
2005-11-21 FIX  : cosmetical changes
